@@ -8,11 +8,47 @@ import sys
 import threading
 from typing import Dict, Optional
 import time  # Added for ETA calculation
+from contextlib import contextmanager
 
 import boto3
 from boto3.s3.transfer import TransferConfig
 from botocore.exceptions import ClientError, NoCredentialsError, PartialCredentialsError
 
+from clearml import Dataset
+
+@contextmanager
+def clearml_client_session(credentials: dict, log_extra={}):
+    """
+    A context manager that creates an explicit, isolated ClearML Session and APIClient
+    based on the provided user credentials.
+    """
+    # 1. Create the explicit Session object with credentials
+    try:
+        from clearml import Task
+        from clearml.backend_api.session import Session
+
+        session = Session(
+        api_key=credentials.get("access_key"),
+        secret_key=credentials.get("secret_key"),
+        # You might need to specify the api_server if not using the default
+        # host="https://api.clear.ml"
+        )
+    
+        # Task.set_credentials(
+        #     key=credentials.get("access_key"),
+        #     secret=credentials.get("secret_key"),
+        #     api_host="http://144.172.105.98:30003",
+        # )
+    except Exception as e:
+        print(e)
+        print("authorization failed", extra=log_extra)
+        raise
+
+    try:
+        # 3. Yield the fully configured client for the 'with' block to use
+        yield
+    finally:
+        print("ClearML session restored and client cleaned up.")
 
 class ProgressPercentage:
     """A thread-safe progress callback for Boto3 uploads with ETA."""
@@ -94,12 +130,18 @@ class S3Uploader:
         aws_secret_access_key: str,
         region_name: str = None,
         endpoint_url: str = None,
+        clearml_access_key: str = None,
+        clearml_secret_key: str = None,
+        clearml_save: bool = True,
+        dataset_name: str = "default",
         verbose: bool = False,
     ):
         if not region_name and not endpoint_url:
             raise ValueError("You must provide either 'region_name' or 'endpoint_url'.")
 
         self.verbose = verbose
+        self.dataset_name = dataset_name
+        self.clearml_save = clearml_save
         try:
             client_kwargs = {
                 "aws_access_key_id": aws_access_key_id,
@@ -110,6 +152,12 @@ class S3Uploader:
             if endpoint_url:
                 client_kwargs["endpoint_url"] = endpoint_url
 
+            self.clearml_credentials = {}
+            if self.clearml_save:
+                self.clearml_credentials["access_key"] = clearml_access_key
+                self.clearml_credentials["secret_key"] = clearml_secret_key
+
+
             self.s3_client = boto3.client("s3", **client_kwargs)
             self.s3_client.list_buckets()  # Verify credentials
 
@@ -138,7 +186,7 @@ class S3Uploader:
     ):
         """Uploads a local folder to an S3 bucket, respecting the sync mode."""
         print(
-            f"\n🚀 Starting upload of '{local_folder_path}' to '{bucket_name}' (Mode: {sync_mode.upper()})..."
+            f"\n Starting upload of '{local_folder_path}' to '{bucket_name}' (Mode: {sync_mode.upper()})..."
         )
         if sync_mode not in self.SYNC_MODES:
             raise ValueError(
@@ -166,7 +214,32 @@ class S3Uploader:
             success = self.upload_file(local_path, bucket_name, s3_key, transfer_config)
             summary["uploaded" if success else "failed"] += 1
 
-        print("\n✨ Upload complete!")
+        print("\nS3 Upload complete!")
+
+        if self.clearml_save:
+
+            with clearml_client_session(self.clearml_credentials):
+
+                print("Creating ClearML dataset...")
+                dataset = Dataset.create(
+                    dataset_name=self.dataset_name,
+                    dataset_project=f"localupload_{self.dataset_name}",
+                    description="Dataset uploaded from local",
+                    # output_uri=s3_main_path
+                )
+
+                # Use ClearML's S3 configuration to point to the external data
+                dataset.add_external_files(
+                    source_url=local_folder_path, dataset_path=self.dataset_name
+                )
+                s3_path = "s3://"+bucket_name+"/"+s3_folder_prefix
+
+                dataset.add_tags(
+                    [f"s3_path:{s3_path}"]
+                )
+
+                dataset.upload()
+                dataset.finalize()
         print(f"Total files processed: {len(files_to_process)}")
         print(f"✅ Successfully uploaded: {summary['uploaded']}")
         print(f"⏭️  Skipped (already synced): {summary['skipped']}")
