@@ -16,6 +16,18 @@ from data.data_validation.config import validation_config
 from data.data_validation.structures import ValidationReport, ValidationResult
 from data.data_validation.validation import DirectoryValidator
 
+# from data.data_ingestion.setup_logging import setup_logging
+
+from datetime import datetime, timezone
+
+# setup_logging(
+#     service_name="s3-handler-service",
+#     es_hosts=["https://144.172.105.98:30092"],
+#     auth=("elastic", "1qaz!QAZ"),
+#     verify_certs=False,
+# )
+
+
 logger = logging.getLogger(__name__)
 
 
@@ -54,7 +66,7 @@ class S3Handler:
             raise HTTPException(status_code=500, detail=f"S3 download error: {e}")
 
     def _validate_and_cleanup_dataset(
-        self, local_file: Path, s3_file_path: str,is_soft: bool=True
+        self, local_file: Path, s3_file_path: str,base_extra: dict ,is_soft: bool=True
     ) -> tuple[str, str, list]:
         """
         Validates a local dataset, removes invalid files, and returns validation status.
@@ -69,9 +81,17 @@ class S3Handler:
         Raises:
             HTTPException: If all files in the dataset fail validation.
         """
-        logger.info(f"Starting validation for dataset at: {local_file}")
-        if local_file.is_file():
-            logger.info(f"Single file detected: {local_file}")
+
+        start_validation_extra = base_extra.copy()
+        start_validation_extra.update({
+            "event":{
+                "category":"validation",
+                "type":"start_validation",
+                "status":"success"
+            }
+        }
+        )
+        logger.info(f"Starting validation for dataset at: {local_file}",extra=start_validation_extra)
 
         # Ensure the validator is configured to unzip archives, which is the default
         validator = DirectoryValidator(config=validation_config, unzip_first=True)
@@ -80,8 +100,8 @@ class S3Handler:
         # check if local_file is a file and convert to ValidationReport
         if isinstance(report, ValidationResult):
             report = ValidationReport(directory_path=local_file, results=[report])
-
-        logger.info(f"Validation report: {report}")
+        
+        
 
         error_details = []
         warning_details = []
@@ -93,11 +113,6 @@ class S3Handler:
             final_message = (
                 "Dataset stored, but some files failed validation and were excluded."
             )
-            logger.warning(
-                f"Validation failed for some files in dataset from S3 path '{s3_file_path}'. "
-                "These files will be excluded."
-            )
-
             invalid_file_paths = set()
             warning_file_paths = set()
 
@@ -159,10 +174,6 @@ class S3Handler:
             logger.error(
                 f"All files in dataset from S3 path '{s3_file_path}' failed validation. Nothing to store."
             )
-            raise HTTPException(
-                status_code=400,
-                detail="All files in the dataset failed validation. No files were stored.",
-            )
 
         return final_status, final_message, error_details,warning_details
 
@@ -229,12 +240,17 @@ class S3Handler:
         user_name: str,
         private: bool,
         dataset_tag: str,
+        base_extra:dict,
+        restructure: bool,
         s3_config_target: dict = None,
         clearml_config: dict = None,
     ) -> dict:
         """
         Download an S3 dataset, persist it to S3-mounted temp directory, and return metadata.
         """
+
+        start_time = datetime.now(timezone.utc)
+        
         try:
             mount_dataset_name = dataset_name or Path(s3_file_path).stem
             storage_handler = DatasetStorageHandler(mount_dataset_name)
@@ -271,7 +287,33 @@ class S3Handler:
                 bucket_name=self.bucket_name,
             )
             print("0" * 10)
-            manager.download(s3_file_path, local_file)
+            try:
+                manager.download(s3_file_path, local_file)
+            except Exception as e:
+                failed_download_extra = base_extra.copy()
+                failed_download_extra.update({
+                "event":{
+                    "category":"s3_handler",
+                    "type":"s3_download",
+                    "status":"failed"
+                }})
+                logger.error(f"error in downloading dataset from source s3 : {e}",extra=failed_download_extra)
+                raise
+
+
+            successful_download_extra = base_extra.copy()
+
+            successful_download_extra.update({
+                "event":{
+                    "category":"s3_handler",
+                    "type":"s3_download",
+                    "status":"success"
+                }
+            }  
+            )
+            logger.info("downloaded source s3 dataset successfully",extra=successful_download_extra)
+
+
             # self.download_file(s3_file_path, local_file, s3_client)
 
             dataset_id = storage_handler.generate_dataset_id()
@@ -290,10 +332,77 @@ class S3Handler:
                 dataset_tag=dataset_tag or "ML",
             )
 
-            # --- Perform validation and cleanup ---
-            final_status, final_message, error_details,warning_details = (
-                self._validate_and_cleanup_dataset(local_file, s3_file_path,is_soft=False)
+            
+            base_extra["dataset"]["id"] = dataset_id
+
+            metadata_extra = base_extra.copy()
+            metadata_extra.update({
+                "event":{
+                    "category":"s3_handler",
+                    "type":"metadata",
+                    "status":"success"
+                }
+            }
             )
+            logger.info("metadata object created",extra=metadata_extra)
+
+            logger.info(f"restructure={restructure}")
+            if restructure==True:
+
+                is_soft = False
+
+                final_status, final_message, error_details,warning_details = (
+                    self._validate_and_cleanup_dataset(local_file, s3_file_path,base_extra,is_soft=is_soft)
+                )
+                validation_results = {
+                    "validation":{
+                        "errors":[],
+                        "warnings":[],
+                        "status":final_status
+                    }
+                }
+                for err in error_details:
+                    for er in err["errors"]:
+                        validation_results["validation"]["errors"].append({
+                            "file":str(err["file"]).split("/")[-1],
+                            "details":er["error"],
+                            "removed":True
+                        })
+
+                for war in warning_details:
+                    for wa in war["warnings"]:
+                        validation_results["validation"]["warnings"].append({
+                            "file":str(war["file"]).split("/")[-1],
+                            "details":wa["warning"],
+                            "removed":not is_soft
+                        })
+                validation_results_extra = base_extra.copy()
+                validation_results_extra.update(validation_results)
+
+                if final_status=="failed":
+                    failed_validation_extra = validation_results_extra.copy()
+                    failed_validation_extra.update({
+                        "event":{
+                        "category":"validation",
+                        "type":"end_validation",
+                        "status":"failed"
+                        }
+                    } 
+                    )
+                    logger.error("validation completely failed (all files removed)",extra=failed_validation_extra)
+                    raise
+                else:
+                    passed_validation_extra = validation_results_extra.copy()
+                    passed_validation_extra.update({
+                        "event":{
+                        "category":"validation",
+                        "type":"end_validation",
+                        "status":"success"
+                        }
+                    } 
+                    )
+
+                    logger.info("validation ended",extra=passed_validation_extra)
 
             acceptable_tags = [
                 "text_generation",
@@ -310,6 +419,7 @@ class S3Handler:
             if (
                 local_file.is_dir()
                 and dataset_tag in acceptable_tags
+                and restructure==True
                 #and not is_structured
             ):
                 restructurer = Restructurer(task_type=dataset_tag)
@@ -321,13 +431,31 @@ class S3Handler:
                     output_path=restructurer_output_path,
                 )
                 if not result:
-                    logger.warning("restructuring failed: skipping this part")
+                    failed_restructuring = base_extra.copy()
+                    failed_restructuring.update({
+                        "event":{
+                            "category":"restructure",
+                            "type":"restructure_result",
+                            "status":"failed"
+                        }
+                    }
+                    )
+                    logger.error("restructuring failed: skipping this part",extra=failed_restructuring)
                 else:
                     local_file = restructurer_output_path
             else:
+                skipping_restructuring = base_extra.copy()
+                skipping_restructuring.update({
+                    "event":{
+                        "category":"restructure",
+                        "type":"skip_restructure",
+                        "status":"failed"
+                    }
+                }
+                )
                 logger.info(
                     f"Skipping restructuring for dataset_tag '{dataset_tag}'. Either not a directory or tag not in {acceptable_tags} or already structured."
-                )
+                    ,extra=skipping_restructuring)
 
             # --- Log remaining files and store the dataset ---
             files_to_upload = (
@@ -337,31 +465,75 @@ class S3Handler:
             )
             logger.info(f"uploading {len(files_to_upload)} files: {files_to_upload}")
 
-            stored_path = storage_handler.store_dataset(
-                local_file,
-                metadata,
-                s3_config=s3_config_target,
-                clearml_config=clearml_config,
+            try:
+
+                stored_path = storage_handler.store_dataset(
+                    local_file,
+                    metadata,
+                    s3_config=s3_config_target,
+                    clearml_config=clearml_config,
+                )
+            except Exception as e:
+                error_saving_extra = base_extra.copy()
+                error_saving_extra.update(
+                    {
+                        "event": {
+                            "category": "s3_handler",
+                            "type": "error_saving",
+                            "status": "failed",
+                        }
+                    }
+                )
+                logger.error(
+                    f"error in saving huggingface dataset: {e}",
+                    extra=error_saving_extra,
+                )
+
+            successfully_stored = base_extra.copy()
+
+            successfully_stored.update({
+                "event":{
+                    "category":"store_dataset",
+                    "type":"end_storing",
+                    "status":"success"
+                }
+            }
             )
 
             # --- Construct final response ---
             response = {
-                "status": final_status,
-                "message": final_message,
+                "status": "success",
                 "dataset_id": dataset_id,
                 "stored_path": stored_path,
             }
-            if error_details:
-                response["validation_errors"] = error_details
 
             return response
-        except HTTPException:
-            raise
         except Exception as e:
-            logger.error(f"Error processing S3 dataset: {e}", exc_info=True)
-            raise HTTPException(
-                status_code=500, detail=f"Internal error processing S3 dataset: {e}"
-            )
+            # 2. Log the failure of the entire process
+            duration_ms = (datetime.now(timezone.utc) - start_time).total_seconds() * 1000
+            process_fail_event = base_extra.copy()
+            process_fail_event["event"] = {
+                "category": "process",
+                "type": "s3_processing_failed",
+                "status": "failure",
+                "duration": int(duration_ms)
+            }
+            # Let the logger format the exception automatically
+            logger.error(f"Failed to process S3 dataset for job {base_extra['trace']['job_id']}", exc_info=True, extra=process_fail_event)
+            
+            # Re-raise the appropriate exception
+            if not isinstance(e, HTTPException):
+                raise HTTPException(status_code=500, detail=str(e))
+            raise
+    
         finally:
-            pass
-            # storage_handler.unmount()
+            # 3. Log the completion of the entire process, including duration
+            duration_ms = (datetime.now(timezone.utc) - start_time).total_seconds() * 1000
+            process_end_event = base_extra.copy()
+            process_end_event["event"] = {
+                "category": "process",
+                "type": "s3_processing_finished",
+                "status": "success", # This block only runs on success
+                "duration": int(duration_ms)
+            }
+            logger.info(f"Finished processing S3 dataset for job {base_extra['trace']['job_id']}", extra=process_end_event)
