@@ -106,8 +106,7 @@ class TabularValidator(BaseValidator):
             # We must collect the result before we can write it
             processed_df = lazy_df.with_columns(replacement_exprs).collect()
 
-            # Determine how to write based on file extension
-            print(processed_df)
+            
             ext = file_path.suffix.lower()
             if ext == ".csv":
                 processed_df.write_csv(file_path)
